;
; HexDump utility written by Chris Vega.
;

;
; Copy both dump.asm and filemap.inc to bin directory of your
; MASM32 package, and execute these commands:
;
;	ml /c /coff /Cp /nologo /I"..\include" dump.asm
;	link /SUBSYSTEM:CONSOLE /OPT:NOREF dump.obj "..\lib\kernel32.lib" "..\lib\user32.lib" "..\lib\masm32.lib"
;

ifndef 	__FILEMAPPING__
		__FILEMAPPING__ 	equ 1

; Prototypes
OpenAndMapFile proto :LPCSTR, :PHANDLE, :PHANDLE, :PHANDLE
CloseAndUnmapFile proto :PHANDLE, :PHANDLE, :PHANDLE


; Maximum length of filename
MAX_FNAME_LEN				equ 256

; Constants
SUCCESS					equ 0
ERROR_OPENFILE_FAILED			equ 1
ERROR_CREATEMAP_FAILED			equ 2
ERROR_MAPVIEW_FAILED			equ 3

; OpenAndMapFile function settings
fm_desired_access			equ GENERIC_READ
fm_share_mode				equ FILE_SHARE_READ
fm_file_attributes			equ FILE_ATTRIBUTE_NORMAL or FILE_ATTRIBUTE_ARCHIVE
fm_page_access				equ PAGE_READONLY
fm_map_desired_access			equ FILE_MAP_READ

;
; This function will try to open and memorymap a specified file, and return
; all the handles for the opened file, created mapping object and the base
; of the memory mapped file. This function returns any of the status defined
; above for errors or success; Also, this function handles the closing
; of the handles for any errors that may happen during the process of memory
; mapping of the file.
;
.code
OpenAndMapFile proc lpszFilename:LPCSTR, _hFile:PHANDLE, _hFileMap:PHANDLE, _hFileBase:PHANDLE
	local dwFileSize:DWORD
	push	edx
	
	; Open the specified file
	invoke	CreateFile, \
				lpszFilename, \
				fm_desired_access, \
				fm_share_mode, \
				NULL, \
				OPEN_EXISTING, \
				fm_file_attributes, \
				NULL
	mov		edx, _hFile
	mov		[edx], eax
	
	.if eax != INVALID_HANDLE_VALUE

		; Create FileMapping object for the file
		push	edx
		invoke	GetFileSize, [edx], NULL
		mov		dwFileSize, eax
		pop		edx
		invoke	CreateFileMapping, \
					[edx], \
					NULL, \
					fm_page_access, \
					0, \
					dwFileSize, \
					NULL
		mov		edx, _hFileMap
		mov		[edx], eax

		.if eax != INVALID_HANDLE_VALUE
		
			; Map the file to the process memory
			invoke	MapViewOfFile, \
					[edx], \
					fm_map_desired_access, \
					0, \
					0, \
					0
			mov	edx, _hFileBase
			mov	[edx], eax
			.if eax != INVALID_HANDLE_VALUE
				mov		eax, SUCCESS
				ret
			.else
				mov	edx, _hFileMap
				invoke	CloseHandle, [edx]
				mov	edx, _hFile
				invoke	CloseHandle, [edx]
				mov	eax, ERROR_MAPVIEW_FAILED
				ret
			.endif
		.else
			mov	edx, _hFile
			invoke	CloseHandle, [edx]
			mov	eax, ERROR_CREATEMAP_FAILED
			ret
		.endif
	.endif

	mov		eax, ERROR_OPENFILE_FAILED
	
	pop		edx
	ret
OpenAndMapFile endp

;
; This function will cloase all the handles opened and created by
; OpenAndMapFile function
;
CloseAndUnmapFile proc _hFile:PHANDLE, _hFileMap:PHANDLE, _hFileBase:PHANDLE
	mov	edx, _hFileBase
	invoke	UnmapViewOfFile, [edx]
	mov	edx, _hFileMap
	invoke	CloseHandle, [edx]
	mov	edx, _hFile
	invoke	CloseHandle, [edx]
	
	ret
CloseAndUnmapFile endp
	
endif